use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::SagePay::API::Object::PaymentMethodSagePay;
use DE_EPAGES::SagePay::API::Payment qw (
    GetAdminURLByCountryCode2
    GetRegisterURLByCountryCode2
    GetSagePayPaymentMethods
    IsOnePaymentMethodInLiveMode
);
use Test::MockObject;
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
             # make sure that the TrustedShops cartridge is installed
            ok(ExistsCartridgeByPackage('DE_EPAGES::SagePay'), 'Cartridge SagePay is installed');
            test_GetLanguageIDByCountryCode2();
            test_GetRegisterURLByCountryCode2();
            test_GetAdminURLByCountryCode2();
            test_GetSagePayPaymentMethods();
            test_IsOnePaymentMethodInLiveMode();
            done_testing();
        });
}
sub test_GetLanguageIDByCountryCode2 {
    ## no critic (ProtectPrivateSubs)
    is(DE_EPAGES::SagePay::API::Payment::_GetLanguageIDByCountryCode2('DE'),1,'GetLanguageIDByCountryCode2 - CountryCode DE gives: - 1');
    is(DE_EPAGES::SagePay::API::Payment::_GetLanguageIDByCountryCode2('FR'),2,'GetLanguageIDByCountryCode2 - Any other CountryCode gives: - 2');
    is(DE_EPAGES::SagePay::API::Payment::_GetLanguageIDByCountryCode2('EN'),2,'GetLanguageIDByCountryCode2 - Any other CountryCode gives: - 2');
    is(DE_EPAGES::SagePay::API::Payment::_GetLanguageIDByCountryCode2(),2,'GetLanguageIDByCountryCode2 - empty CountryCode gives: - 2');
    ## use critic (ProtectPrivateSubs)
}

sub test_GetRegisterURLByCountryCode2{
    is(GetRegisterURLByCountryCode2('DE'), 'https://support.sagepay.com/apply/default.aspx', 'GetRegisterURLByCountryCode2 - check for german register URL');
    is(GetRegisterURLByCountryCode2('FR'), 'https://support.sagepay.com/apply/default.aspx', 'GetRegisterURLByCountryCode2 - check for non german register URL ');
}

sub test_GetAdminURLByCountryCode2{
    my $aPaymentMethods_Test = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
    ];
    is(GetAdminURLByCountryCode2('DE',$aPaymentMethods_Test),'https://test.sagepay.com/mysagepay','GetAdminURLByCountryCode2 - check for URL with German and test mode');
    is(GetAdminURLByCountryCode2('FR',$aPaymentMethods_Test),'https://support.sagepay.com/apply/','GetAdminURLByCountryCode2 - check for URL with non German and test mode');
    my $aPaymentMethods_Live = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay', 'IsInTestMode' => 0}),
    ];
    is(GetAdminURLByCountryCode2('DE',$aPaymentMethods_Live),'https://live.sagepay.com/mysagepay','GetAdminURLByCountryCode2 - check for URL with German and live mode');
    is(GetAdminURLByCountryCode2('FR',$aPaymentMethods_Live),'https://support.sagepay.com/apply/','GetAdminURLByCountryCode2 - check for URL with non German and live mode');
}

sub test_GetSagePayPaymentMethods {
    my $Shop = _CreateShopWith_NoPayments();
    my $SagePayPayments = GetSagePayPaymentMethods($Shop);
    is(scalar @{$SagePayPayments},0,'_CreateServletWith_WithOutPayments - check whats happen if there is no payment - 0');

    $Shop = _CreateShopWith_TwoSagePayPayments_OneOtherPayment();
    $SagePayPayments = GetSagePayPaymentMethods($Shop);
    is(scalar @{$SagePayPayments},2,'_CreateServletWith_TwoSagePayPayments_OneOtherPayment - check if there are only two SagePay payment methods - 2');
    is(@{$SagePayPayments}[0]->alias(),'PaymentMethodSagePay','_CreateServletWith_TwoSagePayPayments_OneOtherPayment - payment one is a - PaymentMethodSagePay');
    is(@{$SagePayPayments}[1]->alias(),'PaymentMethodSagePay','_CreateServletWith_TwoSagePayPayments_OneOtherPayment - payment two is a - PaymentMethodSagePay');
}

sub test_IsOnePaymentMethodInLiveMode {
     _No_PaymentMethod();
    _One_SagePayPayment_InTestMode();
    _One_SagePayPayment_InLiveMode();

    _Two_SagePayPayments_bothInTestMode();
    _Two_SagePayPayments_FirstInTestMode_SecondInLiveMode();
    _Two_SagePayPayments_FirstInLiveMode_SecondInTestMode();
}

sub _No_PaymentMethod{
    my $aPaymentMethods = [];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,0
      ,'_No_Payment - test, if there is no Payment method- 0'
    );
    return;
}

sub _One_SagePayPayment_InTestMode{
    my $aPaymentMethods = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
    ];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,0
        ,'_One_SagePayPayment_InTestMode - test, if the one SagePay Payment method is in test mode - 0'
    );

    return ;
}

sub _One_SagePayPayment_InLiveMode{
    my $aPaymentMethods = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 0}),
    ];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,1
        ,'_One_SagePayPayment_InLiveMode - test, if the one SagePay Payment method is in live mode - 1'
    );
    return ;
}

sub _Two_SagePayPayments_bothInTestMode{
    my $aPaymentMethods = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
    ];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,0
        ,'_Two_SagePayPayments_bothInTestMode - test, if the two SagePay Payment methods both are in test mode - 0'
    );
    return ;
}

sub _Two_SagePayPayments_FirstInTestMode_SecondInLiveMode{
    my $aPaymentMethods = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 0}),
    ];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,1
        ,'_Two_SagePayPayments_FirstInTestMode_SecondInLiveMode - test, if the two SagePay Payment methods, the first is in test mode the second in live mode- 1'
    );
    return ;
}

sub _Two_SagePayPayments_FirstInLiveMode_SecondInTestMode{
    my $aPaymentMethods = [
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 0}),
        _Create_Object_BaseObject({Alias => 'PaymentMethodSagePay','IsInTestMode' => 1}),
    ];
    is(  IsOnePaymentMethodInLiveMode($aPaymentMethods) ,1
        ,'_Two_SagePayPayments_FirstInLiveMode_SecondInTestMode - test, if the two SagePay Payment methods, the first is in live mode the second in test mode- 1'
    );
    return ;
}

sub _Create_Object_BaseObject {
    my ($hAtributes) = @_;
    return  DE_EPAGES::Core::API::Object::BaseObject->new(%{$hAtributes});
}

sub _CreateShopWith_NoPayments{
    return _CreateShop([]);
}

sub _CreateShopWith_TwoSagePayPayments_OneOtherPayment {
    return _CreateShop([
        _CreatePaymentMethodMock('PaymentMethodSagePay'),
        _CreatePaymentMethodMock('NOTCorrectPayment'),
        _CreatePaymentMethodMock('PaymentMethodSagePay'),
    ] );
}

sub _CreateShop {
    my ($aPaymentMethods) = @_;
    my $Shop = Test::MockObject->new();
    $Shop->mock('folder',sub{return $Shop});
    $Shop->mock('children',sub{ return $aPaymentMethods; });
    return $Shop;
}

sub _CreatePaymentMethodMock {
    my ($PaymentMethodName) = @_;
    my $PaymentMethod = Test::MockObject->new();
    $PaymentMethod->mock('class',sub{return $PaymentMethod});
    $PaymentMethod->mock('alias',sub{return $PaymentMethodName});
    return $PaymentMethod;
}
RunScript( Sub => \&Main );