#========================================================================================
# package      DE_EPAGES::ServiRed::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::ServiRed::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::ServiRed::UI::Shop;
use DE_EPAGES::ServiRed::API::Constants qw (
    %SERVIRED_ERRORCODES
    STATUS_NEW
    STATUS_AUTHORISED
    STATUS_CAPTURED
);
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::Exception;
use Test::MockObject::Extends;
use Time::localtime;
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

use strict;

use constant ORDER_ID => 1234;
use constant TRANSACTION_REPEATED_CODE => '0913';
use constant TRANSACTION_SUCCESS_CODE => '0000';
use constant TRANSACTION_AMOUNT => 795;
use constant TEST_SIGNATURE => 'zsOt9rK2EHcCYbe4k9A-8Vwuwuy4Wfgk5fwiTKokz5o=';
use constant TEST_DATETIME => '26/10/2016 14:56';
use constant TEST_AUTH_CODE => '404268';


sub test {
    my $self = shift;

    $self->_test_BacklinkServiRed_WithoutServletParameters();
    $self->_test_BacklinkServiRed_WithoutDsServletParameters_NoLineItemContainerExists();
    $self->_test_BacklinkServiRed_WithoutDsServletParameters_LineItemContainerExists();
    $self->_test_BacklinkServiRed_WithParameters_LineItemContainerExists_NoCustomerOrder();
    $self->_test_BacklinkServiRed_WithParameters_LineItemContainerExists_SuccessfulTransaction();

    $self->_test_ServiRedReturn_WithoutServletParameters();
    $self->_test_ServiRedReturn_NoLineItemContainerExists();
    $self->_test_ServiRedReturn_RepeatedTransaction();
    $self->_test_ServiRedReturn_SignatureMismatch();
    $self->_test_ServiRedReturn_SuccessfulTransaction();

    done_testing();
}


#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_BacklinkServiRed_WithoutServletParameters {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # no Servlet parameters
    my $hServletParams = {
            'default_level' =>  0,
    };
    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 0;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->CallbackServiRedReturn($ServletMock) }, qr/ServiRedCouldNotIdentOrder/, "$SubTestName - \"ServiRedCouldNotIdentOrder\" FORM ERROR is thrown");
}

sub _test_BacklinkServiRed_WithoutDsServletParameters_NoLineItemContainerExists {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # lineItemContainerID Servlet parameter is set but not DS parameters
    my $hServletParams = {
            'default_level' =>  1,
    };
    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 0;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->CallbackServiRedReturn($ServletMock) }, qr/BadCallback_NoPaymentGUID/, "$SubTestName - \"BadCallback_NoPaymentGUID\" FORM ERROR is thrown");
}

sub _test_BacklinkServiRed_WithoutDsServletParameters_LineItemContainerExists {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # lineItemContainerID Servlet parameter is set but not DS parameters
    my $hServletParams = {
            'default_level' =>  1,
    };
    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 1;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->CallbackServiRedReturn($ServletMock) }, qr/BadCallback_NoCallBackParameters/, "$SubTestName - \"BadCallback_NoCallBackParameters\" FORM ERROR is thrown");
}

sub _test_BacklinkServiRed_WithParameters_LineItemContainerExists_NoCustomerOrder {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # all Servlet parameters are set
    my $hServletParams = {
            'default_level' =>  2,
    };
    my $ServletMock = $self->_createServlet($hServletParams, STATUS_NEW);

    my $lineItemContainerExists = 1;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->CallbackServiRedReturn($ServletMock) }, qr/NoAsyncCallReceived/, "$SubTestName - \"NoAsyncCallReceived\" FORM ERROR is thrown");
}

sub _test_BacklinkServiRed_WithParameters_LineItemContainerExists_SuccessfulTransaction {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # all Servlet parameters are set
    my $hServletParams = {
            'default_level' =>  2,
    };
    my $ServletMock = $self->_createServlet($hServletParams, STATUS_AUTHORISED);

    my $lineItemContainerExists = 1;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    $UiShopMock->CallbackServiRedReturn($ServletMock);
    is($ServletMock->vars('ViewObjectID'), ORDER_ID, "$SubTestName - Servlet's ViewObjectID variable gets initialized with Order ID");
}


sub _test_ServiRedReturn_WithoutServletParameters {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # no Servlet parameters
    my $hServletParams = {
            'default_level' =>  0,
    };
    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 0;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->ServiRedReturn($ServletMock) }, qr/ServiRedNoCallBackParameter/, "$SubTestName - \"ServiRedNoCallBackParameter\" ERROR is thrown");
}


sub _test_ServiRedReturn_NoLineItemContainerExists {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # all Servlet parameters are set
    my $hServletParams = {
            'default_level' =>  2,
    };
    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 0;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->ServiRedReturn($ServletMock) }, qr/NoLineItemContainerInShop/, "$SubTestName - \"NoLineItemContainerInShop\" ERROR is thrown");
}

sub _test_ServiRedReturn_RepeatedTransaction {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hServletParams = {
            'default_level' =>  2,
            'extra_parameters' => { 'Ds_Response' => TRANSACTION_REPEATED_CODE },
    }; # all Servlet parameters are set
    my $transactionStatus = STATUS_CAPTURED;
    my $ServletMock = $self->_createServlet($hServletParams, $transactionStatus);

    my $lineItemContainerExists = 1;
    my $hRedSysParamsExtension = { 'Ds_Response' => TRANSACTION_REPEATED_CODE };
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists, $hRedSysParamsExtension);

    $UiShopMock->ServiRedReturn($ServletMock);

    my $MockedShop = $ServletMock->object();
    my $LineItemPaymentMock = $MockedShop->loadObject('ContainerID', 'LineItemContainer')->get('Payment');
    is($LineItemPaymentMock->get('ReturnCode'), undef, "$SubTestName - Callback process gets stopped because of transactions is a repeated one.");
}

sub _test_ServiRedReturn_SignatureMismatch {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hServletParams = {
            'default_level' =>  2,
            'extra_parameters' => { 'Ds_Signature' => 'unmatched_signature_chunk' },
    }; # all Servlet parameters are set

    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 1;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    throws_ok (sub { $UiShopMock->ServiRedReturn($ServletMock) }, qr/ServiRedSignatureMismatch/, "$SubTestName - \"ServiRedSignatureMismatch\" ERROR is thrown");
}

sub _test_ServiRedReturn_SuccessfulTransaction() {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hServletParams = {
            'default_level' =>  2,
    }; # all Servlet parameters are set

    my $ServletMock = $self->_createServlet($hServletParams);

    my $lineItemContainerExists = 1;
    my $UiShopMock = $self->_createUiShop($lineItemContainerExists);

    $UiShopMock->ServiRedReturn($ServletMock);

    my $MockedShop = $ServletMock->object();
    my $LineItemPaymentMock = $MockedShop->loadObject('ContainerID', 'LineItemContainer')->get('Payment');
    my $DateTimeFormatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    my $ExpectedDateTime = $DateTimeFormatter->parse_datetime(TEST_DATETIME, 'datetime', '%d/%m/%Y %H:%M');
    is_deeply(
            [
                $LineItemPaymentMock->get('ReturnCode'),
                $LineItemPaymentMock->get('TransStatus'),
                $LineItemPaymentMock->get('TransTime'),
                $LineItemPaymentMock->get('AuthCode')
            ],
            [
                TRANSACTION_SUCCESS_CODE,
                STATUS_CAPTURED,
                $DateTimeFormatter->parse_datetime($ExpectedDateTime, 'datetime', '%Y-%m-%dT%H:%M:%S'),
                TEST_AUTH_CODE
            ],
            "$SubTestName - LineItemPayment holds right result values."
    );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createServlet {
    my $self = shift;
    my ($hParams, $transactionStatus) = @_;

    my $MockedLineItemContainer = $self->_createLineItemContainer($transactionStatus);
    my $MockedShop = $self->_createShop($MockedLineItemContainer);

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $MockedShop );
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );
    $MockifyServlet->addMockWithReturnValue( 'params', $self->_createServletParams($MockedLineItemContainer, $hParams) );
    $MockifyServlet->addMockWithReturnValueAndParameterCheck( 'registerSession', undef, [ {'string' => 'Test'}, {'int' => 0} ] );

    return $MockifyServlet->getMockObject();
}

sub _createUiShop {
    my $self = shift;
    my ($lineItemContainerExists, $hRedSysParamsExtension) = @_;

    my $UiShopMockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::ServiRed::UI::Shop', [] );

    $UiShopMockify->addMockWithReturnValueAndParameterCheck( '_LogPayment', undef, ['string', 'any'] );
    $UiShopMockify->addMockWithReturnValueAndParameterCheck( '_ExistsObject', $lineItemContainerExists, ['object'] );

    my $RedSysCallbackMock = $self->_createRedSysCallback($hRedSysParamsExtension);
    $UiShopMockify->addMockWithReturnValueAndParameterCheck( '_getRedSysCallbackParamsOrError', $RedSysCallbackMock, ['object', 'object', 'string'] );

    my $DbHandleMock = $self->_createDbHandle();
    $UiShopMockify->addMockWithReturnValue( '_GetCurrentDBHandle', $DbHandleMock );

    return $UiShopMockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $hAttributes = {
        'ID'         => undef,
        'PathString' => undef,
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('loadObject', $LineItemContainer, ['any', 'string']);

    return $MockifyShop->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my ($TransactionStatus) = @_;

    my $hAttributes = {
        'Parent'     => $self->_createCustomerOrder(),
        'PathString' => undef,
        'Payment'    => $self->_createLineItemPayment($TransactionStatus),
    };
    my $MockifyLineItemContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);

    return $MockifyLineItemContainer->getMockObject();
}

sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {
        'CancelledOn' => undef,
        'ID'          => ORDER_ID,
        'PaidOn'      => undef,
        'User'        => 'Test',
    };
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    $MockifyCustomerOrder->addMock('instanceOf', sub {
            shift;
            my ($classString) = @_;
            return ($classString eq 'CustomerOrder') ? 1 : 0;
    } );

    return $MockifyCustomerOrder->getMockObject();
}

sub _createLineItemPayment {
    my $self = shift;
    my ($TransactionStatus) = @_;

    my $hAttributes = {
        'AuthCode'    => undef,
        'IsSecure'    => undef,
        'ReturnCode'  => undef,
        'TransStatus' => $TransactionStatus,
        'TransTime'   => ($TransactionStatus) ? localtime : undef,
    };
    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);

    return $MockedLineItemPayment;
}

sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Presentation::API::Form', {});

    $MockedForm->mock(
        'executeFormError',
        sub {
            shift;
            my ($hFormErrorParams) = @_;
            Error($hFormErrorParams->{'Reason'}, {} );
        }
    );

    return $MockedForm;
}

sub _createRedSysCallback {
    my $self = shift;
    my ($hRedSysParamsExtension) = @_;

    my $RedSysCallbackMockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::ServiRed::API::RedSysCallbackParams', [] );

    $RedSysCallbackMockify->addMockWithReturnValue( 'getJsonMerchantParameters', 'JSON_STRING' );
    $RedSysCallbackMockify->addMockWithReturnValue( 'merchantParametersToHash', $self->_createUnencriptedMerchantParams($hRedSysParamsExtension) );
    $RedSysCallbackMockify->addMock( 'getSignature', sub { return TEST_SIGNATURE; } );

    return $RedSysCallbackMockify->getMockObject();
}

sub _createDbHandle {
    my $self = shift;

    use constant YEAR_OFFSET => 1900;
    my $DbHandleMock = Test::MockObject::Extends->new( 'Time::localtime' );

    $DbHandleMock->mock('currentDateTime', sub {
            my $tm = localtime;
            return sprintf('%02d-%02d-%04d %02d:%02d:%02d', $tm->mday, $tm->mon+1, $tm->year+YEAR_OFFSET, $tm->hour, $tm->min, $tm->sec);
    } );

    return $DbHandleMock;
}

sub _createServletParams {
    my $self = shift;
    my ($LineItemContainer, $hServletParamsRequested) = @_;

    my $servletParamsLevel = $hServletParamsRequested->{'default_level'};

    my $hParams = {};

    if ($servletParamsLevel >= 1) {
        $hParams->{'LineItemContainerID'} = $LineItemContainer;
    }
    if ($servletParamsLevel >= 2) {
        $hParams->{'Ds_MerchantParameters'} = 'eyJEc19EYXRlIjoiMjYlMkYxMCUy..';
        $hParams->{'Ds_Signature'} = TEST_SIGNATURE;
        $hParams->{'Ds_SignatureVersion'} = 'HMAC_SHA256_V1';
    }

    if (defined($hServletParamsRequested->{'extra_parameters'})) {
        my $hExtraParams = $hServletParamsRequested->{'extra_parameters'};
        for my $ParamName (keys %$hParams, keys %$hExtraParams) {
            $hParams->{$ParamName} = $hExtraParams->{$ParamName} // $hParams->{$ParamName};
        }
    }

    return $hParams
}

sub _createUnencriptedMerchantParams {
    my $self = shift;
    my ($hParamsExtension) = @_;

    my $hParams = {};

    $hParams->{'Ds_Amount'} = TRANSACTION_AMOUNT;
    $hParams->{'Ds_AuthorisationCode'} = TEST_AUTH_CODE;
    $hParams->{'Ds_Card_Country'} = '724';
    $hParams->{'Ds_ConsumerLanguage'} = 2;
    $hParams->{'Ds_Currency'} = '978';
    $hParams->{'Ds_Date'} = (split(/ /, TEST_DATETIME))[0];
    $hParams->{'Ds_Hour'} = (split(/ /, TEST_DATETIME))[1];
    $hParams->{'Ds_MerchantCode'} = '297285611';
    $hParams->{'Ds_MerchantData'} = '';
    $hParams->{'Ds_Order'} = '205179426847';
    $hParams->{'Ds_Response'} = TRANSACTION_SUCCESS_CODE;
    $hParams->{'Ds_SecurePayment'} = 1;
    $hParams->{'Ds_Terminal'} = 2;
    $hParams->{'Ds_TransactionType'} = 0;

    if (defined($hParamsExtension)) {
        for my $ParamName (keys %$hParams, keys %$hParamsExtension) {
            $hParams->{$ParamName} = $hParamsExtension->{$ParamName} // $hParams->{$ParamName};
        }
    }

    return $hParams;
}


__PACKAGE__->runScript();

1;
